import logging

import xnat
from textual.app import App, ComposeResult
from textual.binding import Binding
from textual.containers import ScrollableContainer, HorizontalScroll
from textual.widgets import Header, Footer, Label, RichLog
from xnat.exceptions import (XNATAuthError, XNATLoginFailedError, XNATExpiredCredentialsError,
                             XNATNotConnectedError)

from src.log_handler import TextualLogHandler
from src.quit_screen import QuitScreen
from src.xnat_tree import XnatTree


class XnatBrowser(App):
    CSS_PATH = 'browser.tcss'
    BINDINGS = [
        Binding('ctrl+t', 'toggle_dark', 'Toggle dark mode', show=False),
        Binding('q', 'quit', 'Quit'),
        Binding('escape', 'dismiss', 'Dismiss', show=False),
    ]

    def __init__(self, server: str, log_level: int = logging.INFO) -> None:
        super().__init__()
        self.server = server
        self._setup_logging(log_level)

        try:
            self.session = xnat.connect(server=server, default_timeout=300)
        except (XNATAuthError, XNATLoginFailedError, XNATExpiredCredentialsError, XNATNotConnectedError):
            self.logger.error('Error connecting to XNAT server.')

    def _setup_logging(self, log_level: int) -> None:
        self.logger = logging.getLogger('xnat_browser')
        self.log_window = RichLog(id='rich_log', name='Log')
        self.output = Label(id='dicom_info', expand=True)

        log_handler = TextualLogHandler(self.log_window)
        log_handler.setFormatter(
            logging.Formatter('%(asctime)s %(levelname)7s [%(filename)15s:%(lineno)3s - %(funcName)20s()] %(message)s'))

        self.logger.addHandler(log_handler)
        self.logger.setLevel(log_level)

    def compose(self) -> ComposeResult:
        yield Header(show_clock=True)
        with HorizontalScroll(id='h_scroll'):
            yield XnatTree(self.server, self.output, self.logger, id='xnat_tree')
            with ScrollableContainer(id='dicom_info_container'):
                yield self.output
        yield self.log_window
        yield Footer()

    def action_dismiss(self) -> None:
        self.output.update('')

    def action_toggle_dark(self) -> None:
        self.dark = not self.dark

    async def action_quit(self) -> None:
        await self.push_screen(QuitScreen())
